//*********************************************************
//
// Copyright (c) Microsoft. All rights reserved.
// This code is licensed under the MIT License (MIT).
// THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
// IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
// PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

struct PSInput
{
	float4 inVertex : SV_POSITION;
	float2 inTexCoord : TEXCOORD;
	float4 inNormal : COLOR;
};

cbuffer cb0 : register(b0)
{
	float4x4 finalMatrix;
	float4x4 mvMat;
	float4x4 projMat;
	float2 pos;
	float terrainRay;
};

Texture2D texture0 : register(t0);
SamplerState g_sampler : register(s0);

PSInput VSMain(float3 inVertex : POSITION, float2 inTexCoord : TEXCOORD, float3 inNormal : COLOR)
{
	PSInput result;
	
	float3 vertexPos = float3(pos.x+inVertex.x, pos.y+inVertex.y, inVertex.z);
	float distance = length(vertexPos.xy);
	
	float3 newInNormal = inNormal;
	
	if(distance > terrainRay)
	{
		//On deplace le vertex pile au bord du terrain
		float2 newPos = mul(normalize(pos), terrainRay);
		float4x4 trMatrix = float4x4(	1, 0, 0, 	0, 
								0, 1, 0, 	0, 
								0, 0, 1,	0, 
								-pos.x+(newPos.x)+0.5f, -pos.y+(newPos.y)+0.5f, -4, 1);//+0.5 parce que la tile n'est pas centree
								trMatrix = transpose(trMatrix);
		float4x4 mvMatWithTr = mul(mvMat, trMatrix);
		
		//On cherche l'axe de rotation
		float3 rotAxis = normalize(cross(float3(vertexPos.x, vertexPos.y, 0), float3(vertexPos.x, vertexPos.y, -1)));
		float angle = (distance-terrainRay)/4.0 > 3.15 ? 3.15 : (distance-terrainRay)/4.0;//1.57;
		float c = cos(angle);
		float s = sin(angle);
								
		float4x4 rotMatrix = float4x4(	c+(rotAxis.x*rotAxis.x*(1-c)), 			rotAxis.y*rotAxis.x*(1-c)+rotAxis.z*s, 	rotAxis.z*rotAxis.x*(1-c)-rotAxis.y*s, 	0, 
								rotAxis.x*rotAxis.y*(1-c)-rotAxis.z*s, 	c+(rotAxis.y*rotAxis.y*(1-c)), 			rotAxis.z*rotAxis.y*(1-c)+rotAxis.x*s, 	0, 
								rotAxis.x*rotAxis.z*(1-c)+rotAxis.y*s, 	rotAxis.y*rotAxis.z*(1-c)-rotAxis.x*s, 	c+(rotAxis.z*rotAxis.z*(1-c)), 			0, 
								0, 0, 0, 1);
								rotMatrix = transpose(rotMatrix);
								
								
		float4x4 newMvMat = mul(mvMatWithTr, rotMatrix);
		float4x4 unTrMatrix = float4x4(	1, 0, 0, 	0, 
								0, 1, 0, 	0, 
								0, 0, 1,	0, 
								-0.5, -0.5, inVertex.z+4, 1);
								unTrMatrix = transpose(unTrMatrix);
		float4x4 newnewMvMat = mul(newMvMat, unTrMatrix);
		
		float4x4 fm = mul(projMat, newnewMvMat);
		result.inVertex = mul(fm, float4(inVertex, 1.0));
		
		
		newInNormal /= angle > 0.25 ? angle*4 : 1.0;
	}
	else
	{
		float4x4 fm = mul(projMat, mvMat);
		result.inVertex = mul(fm, float4(inVertex, 1.0));
	}
	
	//newInNormal.xy = pos;
	result.inNormal = float4(newInNormal, 1.0);
	result.inTexCoord = inTexCoord;

	return result;
}

float4 PSMain(PSInput input) : SV_TARGET
{
	return input.inNormal*texture0.Sample(g_sampler, input.inTexCoord);
}
