#version 130

uniform mat4 finalMatrix;
uniform vec2 pos;
uniform mat4 projMat;
uniform mat4 mvMat;
uniform float terrainRay;

in vec3 inVertex;
in vec2 inTexCoord;
in vec3 inNormal;
out vec4 color;
out vec2 texCoord;

void main(void) 
{
	vec3 vertexPos = vec3(pos.x+inVertex.x, pos.y+inVertex.y, inVertex.z);
	float distance = length(vertexPos.xy);
	
	vec3 newInNormal = inNormal;
	
	if(distance > terrainRay)
	{
		//On deplace le vertex pile au bord du terrain
		vec2 newPos = normalize(pos)*terrainRay;
		mat4 trMatrix = mat4(	1, 0, 0, 	0, 
								0, 1, 0, 	0, 
								0, 0, 1,	0, 
								-pos.x+(newPos.x)+0.5f, -pos.y+(newPos.y)+0.5f, -4, 1);//+0.5 parce que la tile n'est pas centree
		mat4 mvMatWithTr = mvMat * trMatrix;
		
		//On cherche l'axe de rotation
		vec3 rotAxis = normalize(cross(vec3(vertexPos.x, vertexPos.y, 0), vec3(vertexPos.x, vertexPos.y, -1)));
		float angle = (distance-terrainRay)/4.0 > 3.15 ? 3.15 : (distance-terrainRay)/4.0;//1.57;
		float c = cos(angle);
		float s = sin(angle);
								
		mat4 rotMatrix = mat4(	c+(rotAxis.x*rotAxis.x*(1-c)), 			rotAxis.y*rotAxis.x*(1-c)+rotAxis.z*s, 	rotAxis.z*rotAxis.x*(1-c)-rotAxis.y*s, 	0, 
								rotAxis.x*rotAxis.y*(1-c)-rotAxis.z*s, 	c+(rotAxis.y*rotAxis.y*(1-c)), 			rotAxis.z*rotAxis.y*(1-c)+rotAxis.x*s, 	0, 
								rotAxis.x*rotAxis.z*(1-c)+rotAxis.y*s, 	rotAxis.y*rotAxis.z*(1-c)-rotAxis.x*s, 	c+(rotAxis.z*rotAxis.z*(1-c)), 			0, 
								0, 0, 0, 1);
								
								
								
		mat4 newMvMat = mvMatWithTr * rotMatrix;
		mat4 unTrMatrix = mat4(	1, 0, 0, 	0, 
								0, 1, 0, 	0, 
								0, 0, 1,	0, 
								-0.5, -0.5, inVertex.z+4, 1);
		mat4 newnewMvMat = newMvMat * unTrMatrix;
		
		mat4 fm = projMat * newnewMvMat;
		gl_Position = fm * vec4(inVertex, 1.0);
		
		
		newInNormal /= angle > 0.25 ? angle*4 : 1.0;
	}
	else
	{
		mat4 fm = projMat * mvMat;
		gl_Position = fm * vec4(inVertex, 1.0);
	}
	
	//newInNormal.xy = pos;
	color = vec4(newInNormal, 1.0);
	texCoord = inTexCoord;
}
